@@ -6,6 +6,8 @@ from .types import Diffable
 
 def iapply(obj: Diffable, patches: List[Dict]) -> Diffable:
     """Apply a set of patches to an object, in-place"""
+    if not patches:
+        return obj
     for patch in patches:
         ptr = patch["path"]
         op = patch["op"]
@@ -19,7 +21,11 @@ def iapply(obj: Diffable, patches: List[Dict]) -> Diffable:
             else:  # add/replace
                 parent[key] = value
         elif hasattr(parent, "append"):  # list
-            key = int(key)
+            try:
+                key = int(key)
+            except ValueError:
+                pass
+
             if op == "replace":
                 parent[key] = value
             elif op == "add":
